==> ToolChain_STM32_1.0.49.tar.bz: ToolChain_STM32_1.0.49/Template/examples/example_usart.c <==
/*{ TemplateName::.c ************************************************
 
 Empty template for new c-files.
 Copy and adapt to your needs.
 
}*/

#include "example_usart.h"

//{ Global variables *************************************************

const char* TextHello = "Hello, talk to me!\n";
#define SIZE_YouSaid 20
char YouSaid1[SIZE_YouSaid];
char YouSaid2[SIZE_YouSaid];

BOOL UsartAvailable[2] = {FALSE, FALSE};

#define USART1_INDEX 1                    // using first  USART available on current board
#define USART2_INDEX 2                    // using second USART available on current board

//} Global variables
//{ Function definitions *************************************************

void example_usart_init() {

    u8_t MaxUSART = ttc_usart_get_max_logicalindex();
    Assert(MaxUSART > 0, ec_UNKNOWN); // no USARTs defined by board makefile!
    ttc_usart_errorcode_e Error;

    if (USART1_INDEX <= MaxUSART) { // USART defined by board: initialize it
        ttc_usart_generic_t* Config = ttc_usart_get_configuration(USART1_INDEX);
        Assert(Config, ec_UNKNOWN);

        // change some settings
        Config->receiveBlock = receiveData;
        Config->Flags.Bits.IrqOnRxNE = 1; // enable IRQ on Rx-buffer not empty
        Config->Char_EndOfLine = 13; // packets end at carriage return

        if (0)
            Config->Flags.Bits.DelayedTransmits = 1; // all transmits are queued and sent in interrupt service routin parallel to this task
        else
            Config->Flags.Bits.DelayedTransmits = 0; // all transmits are blocking until data is send

        Error = ttc_usart_init(USART1_INDEX);
        Assert( Error == tue_OK, ec_UNKNOWN);
        UsartAvailable[0] = TRUE;

        // use this USART as default output
        ttc_usart_stdout_set(USART1_INDEX);
    }
    if (USART2_INDEX <= MaxUSART) { // USART defined by board: initialize it
        ttc_usart_generic_t* Config = ttc_usart_get_configuration(USART1_INDEX);
        Assert(Config, ec_UNKNOWN);

        // change some settings
        Config->receiveBlock = receiveData;
        Config->Flags.Bits.IrqOnRxNE = 1; // enable IRQ for incoming data
        Config->Char_EndOfLine = 13; // packets end at carriage return

        Error = ttc_usart_init(USART2_INDEX);
        Assert(Error == tue_OK, ec_UNKNOWN);
        UsartAvailable[1] = TRUE;
    }

    // set first logical USART as standard output (otherwise last initialized one would be used)
    ttc_usart_stdout_set(USART1_INDEX);

    // set usart as stdout interface
    ttc_string_register_stdout(ttc_usart_stdout_send_block, ttc_usart_stdout_send_string);

}
void example_usart_start() {
    ttc_task_create(task_USART,                    // function to start as thread
                "USART",                           // thread name (just for debugging)
                (size_t) 512,                      // stack size
                (void *) NULL,                     // passed as argument to taskLEDs()
                (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                // can return a handle to created task
                );
}
void task_USART(void *pvParameters) {
    pvParameters = pvParameters;      // avoids warning: unused parameter 'pvParameters'

    // set usart device to use as stdout
    example_usart_init(USART1_INDEX);

    u8_t Count = 0;

    while (1) {

        // make use of stdout setting
        ttc_string_printf("Count: %i=0x%x ", Count, Count);
        Count++;

        // zero copying (string is declared as constant)
        // function queues string for transmit and returns immediately
        ttc_usart_send_string_const(USART1_INDEX, TextHello, -1);

        // waits until all bytes have been transmitted (optional)
        ttc_usart_flush_tx(USART1_INDEX);

        // function will wait for all queued strings and return after sending this string
        ttc_usart_send_string_const(USART1_INDEX, " ...", -1);

        if (0 && UsartAvailable[1]) { // other USART initialized: send hello too
            ttc_usart_send_string_const(USART2_INDEX, TextHello, -1);
        }
        ttc_task_check_stack(); // place a breakpoint inside to see real stack-usage; will block forever if stack has overflown

        mSleep(1000);
    }
}
ttc_memory_block_t* receiveData(ttc_usart_generic_t* USART_Generic, ttc_memory_block_t* Data) {
    Assert(USART_Generic != NULL, ec_InvalidArgument);
    Assert(Data          != NULL, ec_InvalidArgument);

    // example implementation: forwarding data USART1<->USART2
    //

    switch (USART_Generic->LogicalIndex) {

    case USART1_INDEX: { // copy data USART1 -> USART2
        if (Data->Size < SIZE_YouSaid-1) {
            ttc_memory_copy(YouSaid1, Data->Buffer, Data->Size);
            Data->Buffer[Data->Size] = 0; // terminate string
        }
        else {
            ttc_memory_copy(YouSaid1, Data->Buffer, SIZE_YouSaid-1);
            Data->Buffer[SIZE_YouSaid-1] = 0; // terminate string
        }

        if (1) {
            u8_t* StrippedBuffer = Data->Buffer;
            Base_t StrippedSize  = Data->Size;
            ttc_string_strip(&StrippedBuffer, &StrippedSize);
            ttc_usart_send_string_const(USART1_INDEX, "relaying: '", -1);
            ttc_usart_send_raw(USART1_INDEX, StrippedBuffer, StrippedSize); // not constant: data will be copied
            ttc_usart_send_string_const(USART1_INDEX, "'\n", -1);
        }
        if (UsartAvailable[1]) { // other USART initialized: copy byte to it

            ttc_usart_send_block(USART2_INDEX, Data);
            Data = NULL;  // memory block now belongs to ttc_usart_send_block()
        }
        break;
    }
    case USART2_INDEX: { // copy data USART2 -> USART1

        ttc_usart_send_block(USART1_INDEX, Data);
        Data = NULL;  // memory block now belongs to ttc_usart_send_block()
        break;
    }
    default: Assert(0, ec_UNKNOWN); break; // should never happen!
    }

    return Data; // memory block can now be reused
}

//} Function definitions

==> ToolChain_STM32_1.0.51.tar.bz: ToolChain_STM32_1.0.51/Template/examples/example_usart.c <==
/*{ TemplateName::.c ************************************************
 
 Empty template for new c-files.
 Copy and adapt to your needs.
 
}*/

#include "example_usart.h"

//{ Global variables *************************************************

const char* TextHello = "Hello, talk to me!\n";
#define SIZE_YouSaid 20
char YouSaid1[SIZE_YouSaid];
char YouSaid2[SIZE_YouSaid];

BOOL UsartAvailable[2] = {FALSE, FALSE};

#define USART1_INDEX 1                    // using first  USART available on current board
#define USART2_INDEX 2                    // using second USART available on current board

//} Global variables
//{ Function definitions *************************************************

void example_usart_init() {

    u8_t MaxUSART = ttc_usart_get_max_logicalindex();
    Assert(MaxUSART > 0, ec_UNKNOWN); // no USARTs defined by board makefile!
    ttc_usart_errorcode_e Error;

    if (USART1_INDEX <= MaxUSART) { // USART defined by board: initialize it
        ttc_usart_generic_t* Config = ttc_usart_get_configuration(USART1_INDEX);
        Assert(Config, ec_UNKNOWN);

        // change some settings
        Config->Flags.Bits.IrqOnRxNE = 1; // enable IRQ on Rx-buffer not empty
        Config->Char_EndOfLine = 13;      // packets end at carriage return

        if (0)
            Config->Flags.Bits.DelayedTransmits = 1; // all transmits are queued and sent in interrupt service routin parallel to this task
        else
            Config->Flags.Bits.DelayedTransmits = 0; // all transmits are blocking until data is send

        Error = ttc_usart_init(USART1_INDEX); // note: first bytes might get lost until ttc_usart_register_receive()
        ttc_usart_register_receive(USART1_INDEX, eu_receiveData, (void*) USART1_INDEX);
        Assert( Error == tue_OK, ec_UNKNOWN);
        UsartAvailable[0] = TRUE;
    }
    if (USART2_INDEX <= MaxUSART) { // USART defined by board: initialize it
        ttc_usart_generic_t* Config = ttc_usart_get_configuration(USART1_INDEX);
        Assert(Config, ec_UNKNOWN);

        // change some settings
        Config->Flags.Bits.IrqOnRxNE = 1; // enable IRQ for incoming data
        Config->Char_EndOfLine = 13; // packets end at carriage return

        // will also initialize USART
        Error = ttc_usart_register_receive(USART2_INDEX, eu_receiveData, (void*) USART2_INDEX);
        Assert(Error == tue_OK, ec_UNKNOWN);
        UsartAvailable[1] = TRUE;
    }

    // set first logical USART as standard output (otherwise last initialized one would be used)
    ttc_usart_stdout_set(USART1_INDEX);

    // set usart as stdout interface
    // ttc_string_register_stdout(ttc_channel_usart_send_block, ttc_usart_stdout_send_string);
    ttc_string_register_stdout(ttc_usart_stdout_send_block, ttc_usart_stdout_send_string);
}
void example_usart_start() {
    ttc_task_create(task_USART,                    // function to start as thread
                "USART",                           // thread name (just for debugging)
                (size_t) 512,                      // stack size
                (void *) NULL,                     // passed as argument to taskLEDs()
                (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                // can return a handle to created task
                );
}
void task_USART(void *pvParameters) {
    pvParameters = pvParameters;      // avoids warning: unused parameter 'pvParameters'

    // set usart device to use as stdout
    example_usart_init(USART1_INDEX);

    u8_t Count = 0;

    while (1) {

        // make use of stdout setting via ttc_string_printf()
        printf("example_usart: %i=0x%x ", Count, Count);
        Count++;

        // zero copying (string is declared as constant)
        // function queues string for transmit and returns immediately
        ttc_usart_send_string_const(USART1_INDEX, TextHello, -1);

        // waits until all bytes have been transmitted (optional)
        ttc_usart_flush_tx(USART1_INDEX);

        // function will wait for all queued strings and return after sending this string
        ttc_usart_send_string_const(USART1_INDEX, " ...", -1);

        if (0 && UsartAvailable[1]) { // other USART initialized: send hello too
            ttc_usart_send_string_const(USART2_INDEX, TextHello, -1);
        }
        ttc_task_check_stack(); // place a breakpoint inside to see real stack-usage; will block forever if stack has overflown

        mSleep(1000);
    }
}
ttc_memory_block_t* eu_receiveData(void* Argument, ttc_usart_generic_t* USART_Generic, ttc_memory_block_t* Data) {
    Assert(USART_Generic != NULL, ec_InvalidArgument);
    Assert(Data          != NULL, ec_InvalidArgument);

    // example implementation: forwarding data USART1<->USART2
    //
    u32_t UsartIndex = (u32_t) Argument;

    switch (UsartIndex) {

    case USART1_INDEX: { // copy data USART1 -> USART2
        if (Data->Size < SIZE_YouSaid-1) {
            ttc_memory_copy(YouSaid1, Data->Buffer, Data->Size);
            Data->Buffer[Data->Size] = 0; // terminate string
        }
        else {
            ttc_memory_copy(YouSaid1, Data->Buffer, SIZE_YouSaid-1);
            Data->Buffer[SIZE_YouSaid-1] = 0; // terminate string
        }

        if (1) {
            u8_t* StrippedBuffer = Data->Buffer;
            Base_t StrippedSize  = Data->Size;
            ttc_string_strip(&StrippedBuffer, &StrippedSize);
            ttc_usart_send_string_const(USART1_INDEX, "relaying: '", -1);
            ttc_usart_send_raw(USART1_INDEX, StrippedBuffer, StrippedSize); // not constant: data will be copied
            ttc_usart_send_string_const(USART1_INDEX, "'\n", -1);
        }
        if (UsartAvailable[1]) { // other USART initialized: copy byte to it

            ttc_usart_send_block(USART2_INDEX, Data);
            Data = NULL;  // memory block now belongs to ttc_usart_send_block()
        }
        break;
    }
    case USART2_INDEX: { // copy data USART2 -> USART1

        ttc_usart_send_block(USART1_INDEX, Data);
        Data = NULL;  // memory block now belongs to ttc_usart_send_block()
        break;
    }
    default: Assert(0, ec_UNKNOWN); break; // should never happen!
    }

    return Data; // memory block can now be reused
}

//} Function definitions
